# -*- coding: utf-8 -*-
from django.shortcuts import render, redirect, get_object_or_404
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.contrib import messages
from lib.access_auth import access_restricted, permission_needed
from lib.tools import confirmation
from covoiturage.models import Annonce
from covoiturage.forms import AnnonceForm, AdminAnnonceForm
from datetime import datetime

ANNONCES_PAR_PAGE = 20


@access_restricted
def liste(request, page=1):
    annonces = Annonce.objects.filter(places__gt=0, date_depart__gt=datetime.now()).order_by("date_depart")
    paginator = Paginator(annonces, ANNONCES_PAR_PAGE)

    try:
        annonces_affiches = paginator.page(page)
    except PageNotAnInteger:
        annonces_affiches = paginator.page(1)
    except EmptyPage:
        annonces_affiches = paginator.page(paginator.num_pages)

    c = {"annonces": annonces_affiches}
    return render(request, "covoiturage/liste.html", c)


@access_restricted
def voir_annonce(request, pk):
    annonce = get_object_or_404(Annonce, pk=pk)
    return render(request, "covoiturage/voir_annonce.html", {"annonce": annonce})


@access_restricted
def poster_annonce(request):
    if request.method == "POST":
        form = AnnonceForm(request.POST)
        if form.is_valid():
            annonce = form.save(commit=False)
            annonce.user = request.user
            annonce.ecole = request.ecole
            annonce.save()
            messages.success(request, "Annonce postée avec succès.")
            return redirect("covoiturage:liste")
    else:
        form = AnnonceForm()

    return render(request, "covoiturage/poster_annonce.html", {"form": form})


@access_restricted
def mes_annonces(request, page=1):
    annonces = Annonce.objects.filter(user=request.user, ecole=request.ecole).order_by("date_depart")
    paginator = Paginator(annonces, ANNONCES_PAR_PAGE)

    try:
        annonces_affiches = paginator.page(page)
    except PageNotAnInteger:
        annonces_affiches = paginator.page(1)
    except EmptyPage:
        annonces_affiches = paginator.page(paginator.num_pages)

    c = {"annonces": annonces_affiches}
    return render(request, "covoiturage/mes_annonces.html", c)


@access_restricted
def editer_annonce(request, pk):
    annonce = get_object_or_404(Annonce.objects.filter(user=request.user, ecole=request.ecole), pk=pk)
    if request.method == "POST":
        form = AnnonceForm(request.POST, instance=annonce)
        if form.is_valid():
            form.save()
            messages.success(request, "Annonce modifiée avec succès.")
            return redirect("covoiturage:mes_annonces")
    else:
        form = AnnonceForm(instance=annonce)

    return render(request, "covoiturage/editer_annonce.html", {"form": form})


@permission_needed("gerer_covoiturage")
def admin_liste(request, page=1):
    annonces = Annonce.objects.filter(ecole=request.ecole).order_by("-date_publication")
    paginator = Paginator(annonces, ANNONCES_PAR_PAGE)

    try:
        annonces_affiches = paginator.page(page)
    except PageNotAnInteger:
        annonces_affiches = paginator.page(1)
    except EmptyPage:
        annonces_affiches = paginator.page(paginator.num_pages)

    c = {"annonces": annonces_affiches}
    return render(request, "covoiturage/admin_liste.html", c)


@permission_needed("gerer_covoiturage")
def admin_editer_annonce(request, pk):
    annonce = get_object_or_404(Annonce.objects.filter(ecole=request.ecole), pk=pk)
    if request.method == "POST":
        form = AdminAnnonceForm(request.POST, instance=annonce)
        if form.is_valid():
            form.save()
            messages.success(request, "Annonce modifiée avec succès.")
            return redirect("gestion:covoiturage:admin_liste")
    else:
        form = AdminAnnonceForm(instance=annonce)

    return render(request, "covoiturage/admin_editer_annonce.html", {"form": form})


@permission_needed("gerer_covoiturage")
def admin_supprimer(request, pk):
    annonce = get_object_or_404(Annonce.objects.filter(ecole=request.ecole), pk=pk)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            annonce.delete()
            messages.success(request, "Annonce supprimée avec succès.")
        return redirect("gestion:covoiturage:admin_liste")

    message = "Êtes-vous sûr de vouloir supprimer cette annonce ?"
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer une formation")
